(* Translate a program in AST form into a java program.  Throw an
    exception if something is wrong, e.g., a reference to an unknown
    variable or function *)

open Ast
open Sast

let imports = "import java.awt.Graphics;\n" ^ "import java.awt.Point;\n" ^ "import java.awt.geom.*;\n" ^ "import java.awt.Graphics2D;\n"

let default_paint = "public void paint(Graphics g)"

let default_keyword = "public void "(*put before  global variables and functions*)
let default_implicit_param = "null"		(*for null declaration*)

let main_func = "public static void main(String args[]) {\n" ^ "test t = new test();\n" ^ "t.setVisible(true);\n"
    ^ "t.setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);\n" ^ "t.setSize(600, 600);\n" ^ "}\n"

(*translate binary operators*)
let string_of_operator = function
	  Add -> "+"    	
	| Sub -> "-"  
	| Mult -> "*"      	
	| Div -> "/"
    | Equal -> "==" 			
    | Neq -> "!=" 
    | Less -> "<" 	
    | Leq -> "<=" 
	| Greater -> ">" 
	| Geq -> ">=" 
	
(*translate types*)
let string_of_type =function 
      Int ->"int"
    | String -> "String"
    | Point -> "Point2D"
    | Line -> "Line2D"
    | Ellipse -> "Ellipse2D"
    | Curve -> "Arc2D"
    | Boolean -> "Boolean"
    


(*translate expression*)
let rec string_of_expr = function
    |IntLiteral(l) -> string_of_int l
    |StringLiteral(l) -> "\"" ^ l ^ "\""
    |MLine(e1, e2, e3, e4) -> ".setLine(" ^ string_of_expr e1 ^ "," ^ string_of_expr e2 ^ "," ^ string_of_expr e3 ^ "," ^ string_of_expr e4 ^ ")"
    |MEllipse(e1, e2, e3, e4) -> ".setFrame(" ^ string_of_expr e1 ^ "," ^ string_of_expr e2 ^ "," ^ string_of_expr e3 ^ "," ^ string_of_expr e4 ^ ")"
    |MCurve(e1, e2, e3, e4, e5, e6) -> ".setArc(" ^ string_of_expr e1 ^ "," ^ string_of_expr e2 ^ "," ^ string_of_expr e3 ^ "," ^ string_of_expr e4 ^ "," ^ string_of_expr e5 ^ "," ^ string_of_expr e6 ^ ", Arc2D.OPEN)"
    |Id(s) -> s
    |MPoint(e1, e2) -> ".setLocation(" ^ string_of_expr e1 ^ "," ^ string_of_expr e2 ^ ")"
    |GetX(e) -> "(int) " ^ string_of_expr e ^ ".getX()"
    |GetY(e) -> "(int) " ^ string_of_expr e ^ ".getY()"
    |Binop(e1, op, e2) -> string_of_expr e1 ^ " " ^ string_of_operator(op) ^ " " ^ string_of_expr e2 
    |Assign(s, MLine(e1, e2, e3, e4)) -> string_of_expr s ^ ".setLine(" ^ string_of_expr e1 ^ "," ^ string_of_expr e2 ^ "," ^ string_of_expr e3 ^ "," ^ string_of_expr e4 ^ ")"
    |Assign(s, MPoint(e1, e2)) -> string_of_expr s ^ ".setLocation(" ^ string_of_expr e1 ^ "," ^ string_of_expr e2 ^ ")"
    |Assign(s, MEllipse(e1, e2, e3, e4)) -> string_of_expr s ^ ".setFrame(" ^ string_of_expr e1 ^ "," ^ string_of_expr e2 ^ "," ^ string_of_expr e3 ^ "," ^ string_of_expr e4 ^ ")"
    |Assign(s, MCurve(e1, e2, e3, e4, e5, e6)) -> string_of_expr s ^ ".setArc(" ^ string_of_expr e1 ^ "," ^ string_of_expr e2 ^ "," ^ string_of_expr e3 ^ "," ^ string_of_expr e4 ^ "," ^ string_of_expr e5 ^ "," ^ string_of_expr e6 ^ ", Arc2D.OPEN)"
    |Assign(s, e) -> string_of_expr s ^ " = " ^ string_of_expr e
    |Call(s, e_list) -> s ^ "(" ^ String.concat "," (List.map string_of_expr e_list) ^ ")"    |_ -> "never get there!"

(*translate the expression from Sast*)
let rec string_of_exprs = function
    | ExprType(e,_) -> string_of_expr e
    
(*translation of init from Sast*)
let string_of_init = function 
    | NullInit(Point, name) -> "Point2D " ^ name ^ " = new Point2D.Float()"
    | NullInit(Line, name) -> "Line2D " ^ name ^ " = new Line2D.Float()"
    | NullInit(Ellipse, name) -> "Ellipse2D " ^ name ^ " = new Ellipse2D.Float()"
    | NullInit(Curve, name) -> "Arc2D " ^ name ^ " = new Arc2D.Float()"
    | NullInit(t, s) -> string_of_type t ^ " " ^ s 
    | VarInit(Point, name, MPoint(e1, e2)) -> "Point2D " ^ name ^ " = new Point2D.Float();\n" 
    	^ name ^ ".setLocation(" ^ string_of_expr e1 ^ ", " ^ string_of_expr e2 
    | VarInit(Line, name, MLine(e1, e2, e3, e4)) -> "Line2D" ^ name ^ " = new Line2D.Float();\n"
    	^ name ^ ".setLine(" ^ string_of_expr e1 ^ ", " ^ string_of_expr e2 ^ ", " ^ string_of_expr e3 ^ ", " ^ string_of_expr e4 
    | VarInit(Ellipse, name, MEllipse(e1, e2, e3, e4)) -> "Ellipse2D" ^ name ^ " = new Ellipse2D.Float();\n"
    | VarInit(Curve, name, MCurve(e1, e2, e3, e4, e5, e6)) -> "Arc2D" ^ name ^ " = new Arc2D.Float();\n"
    | VarInit(t, s, e) -> string_of_type t ^ " " ^ s ^ " = " ^ string_of_expr e 

let rec string_of_inits = function
    SInit(i) -> string_of_init (i)
     	


(*translate statement*)
let rec string_of_stmt = function
    |SBlock(stmts) -> "{\n" ^ String.concat "" (List.map string_of_stmt stmts)^ "}\n"
    |SExpr(e) -> string_of_exprs e ^ ";\n"
    |SNoElseMark -> "{;}\n"
    |SReturn(e) -> "return " ^ string_of_exprs e ^ ";\n"
    |SIf(e, s1, SNoElseMark) -> "if (" ^ string_of_exprs e ^ ")\n" ^ string_of_stmt s1
    |SIf(e, s1, s2) -> "if (" ^ string_of_exprs e ^ ")\n" ^ string_of_stmt s1 ^ "else\n" ^ string_of_stmt s2
    |SFor(e1, e2, e3, stmt) -> "for (" ^ string_of_exprs e1  ^ ";" ^string_of_exprs e2 ^ ";" ^ string_of_exprs e3 ^ ")\n" ^ string_of_stmt stmt ^ "\n"
    |SWhile(e, s) -> "while (" ^ string_of_exprs e ^ ")\n" ^ string_of_stmt s ^ ";\n"
    |SPrint(e) -> "System.out.println(" ^ string_of_exprs e ^ ");\n"
    |SVarDecl(l)-> string_of_inits l ^ ";\n"
    |SDraw(e) -> "g2.draw(" ^ string_of_exprs e ^ ");\n"

    
(*translation of variable signature*)
let string_of_varsig = function 
	VarSig (t, id) ->string_of_type t ^ " " ^ id
	
    
(*translate function declairation*)
let string_of_func_decl func_decl =
    match func_decl.sfname with
      | "basic" -> "public void basic()\n{\n" ^ string_of_stmt func_decl.sbody^ "}\n"
      | "paint" -> "public void paint(Graphics g)\n{\nGraphics2D g2 = (Graphics2D) g;\n" ^ string_of_stmt func_decl.sbody ^ "}\n"
      | _ ->  "public " ^ string_of_type func_decl.sreturn ^ " " ^ func_decl.sfname ^ "(" ^ (String.concat "," (List.map string_of_varsig func_decl.sformals))
		^ ")\n{\n" ^ string_of_stmt func_decl.sbody ^ "}\n"



(*translation of constructs*)
let string_of_construct =function 
    SGlobalVar(l)->"public static " ^ string_of_inits l ^ ";\n"
  | SFuncDef (f)->string_of_func_decl f

(*translation of construct list and the generate whole java code*) 
let string_of_class class_name programs = 
	"\n" ^ imports ^ "\n" ^ "\npublic class " ^ class_name ^ " extends javax.swing.JFrame{\n" ^ String.concat "\n" (List.map string_of_construct programs) ^ "public static void main(String args[]){\n" ^ class_name ^ " t = new " ^ class_name ^ "();\nt.setVisible(true);\nt.basic();\nt.setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);\nt.setSize(600,600);\n}\n}\n" 

	
