(* parser_test.ml - This is used for testing purposes.     *)
(* It outputs the results of an AST in string-readable form         *)

open Ast

let string_of_type = function
    | Int -> "TYPE_INT"
    | Point -> "TYPE_POINT"
    | Line -> "TYPE_LINE"
    | Ellipse -> "TYPE_ELLIPSE"
    | Curve -> "TYPE_CURVE"
    (*| LAYER -> "TYPE_LAYER"*)
    | String -> "TYPE_STRING"
    | Void -> "TYPE_VOID"
    | Boolean -> "TYPE_BOOLEAN"


let string_of_binop = function
   Add -> "ADD"
 | Sub -> "SUB"
 | Mult -> "MULT"
 | Div -> "DIV"
 | Equal -> "EQUAL"
 | Neq -> "NEQ"
 | Less-> "LT"
 | Leq -> "LTE"
 | Greater -> "GT"
 | Geq -> "GTE"

let rec string_of_expr = function
    IntLiteral(l) -> "EXPR_LITERAL:[LITERAL_INT:[" ^ (string_of_int l) ^ "]]"
  | StringLiteral(l) -> "EXPR_LITERAL:[LITERAL_STRING:[" ^ l ^ "]]"
  | Id(s) -> "EXPR_ID:[" ^ s ^ "]"
  (*| Dotop(s, op, e) -> "EXPR_DOTOP[" ^ s ^ "," ^ string_of_binop op ^ "," string_of_expr e ^"]"*)
  | MPoint(e1, e2) -> "EXPR_POINT[" ^ string_of_expr e1 ^ "," ^ string_of_expr e2 ^"]"
  | MLine(e1, e2, e3, e4) -> "EXPR_LINE[" ^ string_of_expr e1^ "," ^ string_of_expr e2 ^ "," ^ string_of_expr e3 ^ "," ^ string_of_expr e4 ^"]"
  | MEllipse(e1, e2, e3, e4) -> "EXPR_ELLIPSE[" ^ string_of_expr e1^ "," ^ string_of_expr e2 ^ "," ^ string_of_expr e3 ^ "," ^ string_of_expr e4 ^"]"
  | MCurve(e1, e2, e3, e4, e5, e6) -> "EXPR_CURVE[" ^ string_of_expr e1^ "," ^ string_of_expr e2 ^"," ^string_of_expr e3^ "," ^ string_of_expr e4 ^string_of_expr e5^ "," ^ string_of_expr e6 ^ "]"
  | GetX(e) -> "EXPR_GETX:[" ^ string_of_expr e ^ "]"
  | GetY(e) -> "EXPR_GETY:[" ^ string_of_expr e ^ "]"
(*possibly layer*)
  | Binop(e1, op, e2) -> "EXPR_BINOP:[" ^ string_of_expr e1 ^ "," ^ string_of_binop op ^ "," ^ string_of_expr e2 ^ "]"
  | Assign(e1, e2) -> "EXPR_ASSIGN:[" ^ string_of_expr e1 ^ "," ^ string_of_expr e2 ^ "]"
  | Call(f, el) ->
      "EXPR_CALL:[" ^ f ^ ",LIST:[" ^ String.concat ", " (List.map string_of_expr el) ^ "]]"

let string_of_init = function
    NullInit(t, s) -> "INIT_NULL:[" ^ string_of_type t ^ "," ^ s ^ "]"
  | VarInit(t,s,e) -> "INIT_VAR:[" ^ string_of_type t ^ "," ^ s ^ "," ^ (string_of_expr e) ^ "]"
 
let rec string_of_stmt = function
    Block(stmts) ->
      "STMT_BLOCK:[LIST:[" ^ String.concat "," (List.map string_of_stmt stmts) ^ "]]"
  | Expr(expr) -> "STMT_EXPR:[" ^ string_of_expr expr ^ "]";
  | Return(expr) -> "STMT_RETURN:[" ^ string_of_expr expr ^ "]";
  | Print(expr) -> "STMT_PRINT:[" ^ string_of_expr expr ^ "]";
  | If(e, s, NoElseMark) -> "STMT_IF_NOELSE:[" ^ string_of_expr e ^ "," ^ string_of_stmt s ^ "]"
  | If(e, s1, s2) ->  "STMT_IF:[" ^ string_of_expr e ^ "," ^
      string_of_stmt s1 ^ "," ^ string_of_stmt s2 ^ "]"
  | For(e1, e2, e3, s) ->
      "STMT_FOR:[" ^ string_of_expr e1  ^ "," ^ string_of_expr e2 ^ "," ^
      string_of_expr e3  ^ "," ^ string_of_stmt s ^ "]"
  | While(e, s) -> "STMT_WHILE:[" ^ string_of_expr e ^ "," ^ string_of_stmt s ^ "]"
  | NoElseMark -> "STMT_NOELSEMARK"
  | VarDecl(s)->string_of_init s
  | Draw(e) -> "STMT_DRAW[" ^ string_of_expr e ^ "]"

let string_of_varSig = function
  VarSig(t, s) -> "VARSIG:[" ^ string_of_type t ^ "," ^ s ^ "]"

let string_of_func_decl fdecl =
  "FUNCDECL:[FUNTYPE[" ^ string_of_type fdecl.return ^ "],FUNNAME[" ^ fdecl.fname ^ "],LIST:[" ^
  String.concat "," (List.map string_of_varSig fdecl.formals) ^ "],LIST:[" ^
  string_of_stmt fdecl.body ^ "]]"

let string_of_construct = function
    GlobalVar(i) -> "CONSTRUCT_GLOBALVAR:[" ^ (string_of_init i) ^ "]"
  | FuncDef(f) -> "CONSTRUCT_FUNCDEF:[" ^ (string_of_func_decl f) ^ "]"

let string_of_program construct_list =
  "PROGRAM:[LIST:[" ^ (String.concat "," (List.map string_of_construct construct_list)) ^ "]]\n"
